import re
from mext.uri.schemes import *

__all__ = ['urlsplit']


def urlsplit(url):
    """
        Parse an URL as <scheme>://<netloc></path>?<query>#<fragment>
        Return (scheme, netloc, path, query, fragment).
    """
    #url = unicode(url)
    key = type(url), url
    if key in _parse_cache:
        try:
            return _parse_cache[key]
        except:
            # catch cache cleanup from a different thread
            pass

    if len(_parse_cache) >= MAX_CACHE_SIZE:
        _clean_cache()

    scheme = None
    if ':' in url:
        scheme, path = url.split(':', 1)
        scheme = scheme.lower()
    if scheme in _scheme_rx:
        r = (scheme,) + _scheme_rx[scheme].match(path).groups()
    elif scheme and _rx_scheme.match(scheme):
        r = (scheme, None, path, None, None)
    else:
        r = (None, None) + _rx_relative.match(url).groups()
    _parse_cache[key] = r
    return r





def _gen_re_uri_noscheme(netloc, query, frag):
    frag_part = query_part = '()'
    next_seps = []
    if frag:
        frag_part = _gen_re_part('#', next_seps)
    if query:
        query_part = _gen_re_part('?', next_seps)
    rtail = r'(%s*)%s%s' % (_gen_re_not_sep(next_seps), query_part, frag_part)
    if netloc:
        next_seps.insert(0, '/')
        return r'//(%s+)%s' % (_gen_re_not_sep(next_seps), rtail)
    elif netloc is None: #relative
        return rtail
    else:
        return r'()' + rtail



def _gen_re_part(sep, next_seps):
    next_seps.insert(0, sep)
    return r'(?:\%s(%s*))?' % (sep, _gen_re_not_sep(next_seps[1:]))

def _gen_re_not_sep(next_seps):
    return '[^%s]' % ''.join(next_seps) if next_seps else '.'








_scheme_rx = {}

def reg_scheme(scheme):
    if scheme in _scheme_rx:
        return
    pattern = _gen_re_uri_noscheme(
        scheme in scheme_uses_netloc,
        scheme in scheme_uses_query,
        scheme in scheme_uses_fragment
    )
    _scheme_rx[scheme] = re.compile(pattern)

map(reg_scheme, scheme_uses_netloc)
map(reg_scheme, scheme_uses_query)
map(reg_scheme, scheme_uses_fragment)

_rx_http = _scheme_rx['http']
#_rx_default = re.compile(_gen_re_uri_noscheme('[^:]+', False, False, False))
_rx_relative = re.compile(_gen_re_uri_noscheme(None, True, True))
_rx_scheme = re.compile('[a-z+-.]+')

## Scheme names consist of a sequence of characters. The lower case
## letters "a"--"z", digits, and the characters plus ("+"), period
## ("."), and hyphen ("-") are allowed. For resiliency, programs
## interpreting URLs should treat upper case letters as equivalent to
## lower case in scheme names (e.g., allow "HTTP" as well as "http").









##
## urlsplit cache
##

_parse_cache = {}
_clear_half = True
MAX_CACHE_SIZE = 500

def _clean_cache():
    global _clear_half
    keys = list(_parse_cache)
    _clear_half = not _clear_half
    idx_half = len(keys)/2
    if _clear_half:
        keys = keys[:idx_half]
    else:
        keys = keys[idx_half:]
    map(_parse_cache.__delitem__, keys)
    del keys



